"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/llm.index.ts
var llm_index_exports = {};
__export(llm_index_exports, {
  CompletionBuilder: () => CompletionBuilder,
  assistantMsg: () => assistantMsg,
  completion: () => completion,
  filePart: () => filePart,
  systemMsg: () => systemMsg,
  userMsg: () => userMsg
});
module.exports = __toCommonJS(llm_index_exports);

// src/completion.builder.ts
var import_resolved = require("@synstack/resolved");
var import_ai = require("ai");
var CompletionBuilder = class _CompletionBuilder {
  /**
   * Create a new CompletionBuilder with an empty configuration.
   */
  static get new() {
    return new _CompletionBuilder({});
  }
  /**
   * Create a new CompletionBuilder from an existing configuration.
   */
  static from(options) {
    return new _CompletionBuilder(
      options ?? {}
    );
  }
  _options;
  constructor(options) {
    this._options = options;
  }
  /**
   * Merge current options with another configuration
   */
  merge(options) {
    const newOptions = options instanceof _CompletionBuilder ? options._options : options;
    return new _CompletionBuilder({
      ...this._options,
      ...newOptions
    });
  }
  /**
   * Get the current options as a plain object.
   */
  get options() {
    return this._options;
  }
  /**
   * Get the current options as a plain object.
   */
  valueOf() {
    return this._options;
  }
  // #region Model
  /**
   * Set the language model to use.
   */
  model(model) {
    return this.merge({ model });
  }
  /**
   * Set the middlewares to wrap the model with.
   */
  middlewares(middlewares) {
    return this.merge({ middlewares });
  }
  /**
   * Append a middleware to the existing middlewares.
   *
   * Appended middlewares will be executed after the existing middlewares.
   */
  appendMiddlewares(middlewares) {
    return this.merge({
      middlewares: [...this._options.middlewares ?? [], ...middlewares]
    });
  }
  /**
   * Prepend a middleware to the existing middlewares.
   *
   * Prepended middlewares will be executed before the existing middlewares.
   */
  prependMiddlewares(middlewares) {
    return this.merge({
      middlewares: [...middlewares, ...this._options.middlewares ?? []]
    });
  }
  // #endregion
  // #region Messages
  /**
   * Set the messages to use in the prompt.
   */
  messages(messages) {
    return this.merge({ messages: (0, import_resolved.pipe)(messages).$ });
  }
  /**
   * Append messages to the existing prompt messages.
   */
  appendMessages(messages) {
    return this.merge({
      messages: (0, import_resolved.pipe)([
        this._options.messages ?? [],
        (0, import_resolved.pipe)(messages).$
      ])._(([oldMsgs, newMsgs]) => [...oldMsgs, ...newMsgs]).$
    });
  }
  // #endregion
  // #region Tools
  /**
   * Set the tools that the model can call. The model needs to support calling tools.
   */
  tools(tools) {
    return this.merge({ tools });
  }
  /**
   * Set the tool choice strategy. Default: 'auto'.
   *
   * Supports the following settings:
   * - `auto` (default): the model can choose whether and which tools to call.
   * - `required`: the model must call a tool. It can choose which tool to call.
   * - `none`: the model must not call tools
   * - `{ type: 'tool', toolName: string (typed) }`: the model must call the specified tool
   */
  toolChoice(toolChoice) {
    return this.merge({
      toolChoice
    });
  }
  /**
   * Set a function that attempts to repair a tool call that failed to parse.
   * @experimental
   */
  repairToolCalls(repairToolCalls) {
    return this.merge({ experimental_repairToolCalls: repairToolCalls });
  }
  /**
   * Limit the tools that are available for the model to call without
   * changing the tool call and result types in the result.
   * @experimental
   */
  activeTools(activeTools) {
    return this.merge({ experimental_activeTools: activeTools });
  }
  // #endregion
  // #region LLM
  /**
   * Maximum number of tokens to generate.
   */
  maxTokens(maxTokens) {
    return this.merge({ maxTokens });
  }
  /**
   * Temperature setting.
   * The value is passed through to the provider. The range depends on the provider and model.
   * It is recommended to set either `temperature` or `topP`, but not both.
   */
  temperature(temperature) {
    return this.merge({ temperature });
  }
  /**
   * Nucleus sampling.
   * The value is passed through to the provider. The range depends on the provider and model.
   * It is recommended to set either `temperature` or `topP`, but not both.
   */
  topP(topP) {
    return this.merge({ topP });
  }
  /**
   * Only sample from the top K options for each subsequent token.
   * Used to remove "long tail" low probability responses.
   * Recommended for advanced use cases only. You usually only need to use temperature.
   */
  topK(topK) {
    return this.merge({ topK });
  }
  /**
   * Frequency penalty setting.
   * It affects the likelihood of the model to repeatedly use the same words or phrases.
   * The value is passed through to the provider. The range depends on the provider and model.
   */
  frequencyPenalty(frequencyPenalty) {
    return this.merge({ frequencyPenalty });
  }
  /**
   * Presence penalty setting.
   * It affects the likelihood of the model to repeat information that is already in the prompt.
   * The value is passed through to the provider. The range depends on the provider and model.
   */
  presencePenalty(presencePenalty) {
    return this.merge({ presencePenalty });
  }
  /**
   * The seed (integer) to use for random sampling.
   * If set and supported by the model, calls will generate deterministic results.
   */
  seed(seed) {
    return this.merge({ seed });
  }
  // #endregion
  // # region Flow
  /**
   * Stop sequences.
   * If set, the model will stop generating text when one of the stop sequences is generated.
   */
  stopSequences(stopSequences) {
    return this.merge({ stopSequences });
  }
  /**
   * Maximum number of retries. Set to 0 to disable retries. Default: 2.
   */
  maxRetries(maxRetries) {
    return this.merge({ maxRetries });
  }
  /**
   * An optional abort signal that can be used to cancel the call.
   */
  abortSignal(abortSignal) {
    return this.merge({ abortSignal });
  }
  /**
   * Maximum number of sequential LLM calls (steps), e.g. when you use tool calls. Must be at least 1.
   *
   * A maximum number is required to prevent infinite loops in the case of misconfigured tools.
   *
   * By default, it's set to 1, which means that only a single LLM call is made.
   */
  maxSteps(maxSteps) {
    return this.merge({ maxSteps });
  }
  /**
   * When enabled, the model will perform additional steps if the finish reason is "length" (experimental).
   *
   * By default, it's set to false.
   * @experimental
   */
  continueSteps(continueSteps) {
    return this.merge({ experimental_continueSteps: continueSteps });
  }
  // #endregion
  // #region Experimental
  /**
   * Optional telemetry configuration.
   * @experimental
   */
  telemetry(telemetry) {
    return this.merge({ experimental_telemetry: telemetry });
  }
  /**
   * Additional provider-specific metadata. They are passed through
   * to the provider from the AI SDK and enable provider-specific
   * functionality that can be fully encapsulated in the provider.
   * @experimental
   */
  providerMetadata(providerMetadata) {
    return this.merge({ experimental_providerMetadata: providerMetadata });
  }
  // public output(output: "object" | "array" | "no-schema") {
  //   return this.merge({ output });
  // }
  // #endregion
  // #region Generate
  get modelWithMiddlewares() {
    if (!this._options.middlewares) return this._options.model;
    return this._options.middlewares.reduce(
      (model, middleware) => middleware(model),
      this._options.model
    );
  }
  /**
   * Generate a text and call tools for a given prompt using a language model.
   *
   * This function does not stream the output. If you want to stream the output, use `streamText` instead.
   *
   * @returns A result object that contains the generated text, the results of the tool calls, and additional information.
   */
  async generateText() {
    const resolvedConfig = await (0, import_resolved.pipe)(this._options.messages)._((messages) => ({
      ...this._options,
      model: this.modelWithMiddlewares,
      messages
    })).$;
    const res = await (0, import_ai.generateText)(resolvedConfig);
    return res;
  }
  /**
   * Generate a text and call tools for a given prompt using a language model.
   *
   * This function streams the output. If you do not want to stream the output, use `generateText` instead.
   *
   * @returns A result object for accessing different stream types and additional information.
   */
  streamText() {
    return (0, import_resolved.pipe)(this._options.messages)._(
      (msgs) => (0, import_ai.streamText)({
        ...this._options,
        model: this.modelWithMiddlewares,
        messages: msgs
      })
    ).$;
  }
  /**
   * Generate a structured, typed object for a given prompt and schema using a language model.
   *
   * This function does not stream the output. If you want to stream the output, use `streamObject` instead.
   *
   * @returns A result object that contains the generated object, the finish reason, the token usage, and additional information.
   */
  generateObject(options) {
    return (0, import_resolved.pipe)(this._options.messages)._(
      (msgs) => (0, import_ai.generateObject)({
        ...this._options,
        messages: msgs,
        model: this.modelWithMiddlewares,
        schema: options.schema,
        schemaName: options.name,
        schemaDescription: options.description
      })
    ).$;
  }
  /**
   * Generate a structured, typed object for a given prompt and schema using a language model.
   *
   * This function streams the output. If you do not want to stream the output, use `generateObject` instead.
   *
   * @returns A result object for accessing the partial object stream and additional information.
   */
  streamObject(options) {
    return (0, import_resolved.pipe)(this._options.messages)._(
      (msgs) => (0, import_ai.streamObject)({
        ...this._options,
        model: this.modelWithMiddlewares,
        messages: msgs,
        schema: options.schema,
        schemaName: options.name,
        schemaDescription: options.description
      })
    ).$;
  }
  // #endregion
};
var completion = CompletionBuilder.new;

// src/llm.types.ts
var import_resolved2 = require("@synstack/resolved");
var import_zod = require("zod");

// src/message.builder.ts
var import_resolved3 = require("@synstack/resolved");
var import_ts_pattern = require("ts-pattern");
var import_text = require("@synstack/text");
var import_fs = require("fs");
function mapTextPart(v) {
  return { type: "text", text: v };
}
function mapImagePart(v) {
  return {
    type: "image",
    mimeType: v.mimeType,
    image: v.encoding === "path" ? (0, import_fs.readFileSync)(v.data) : v.data
  };
}
function mapFilePart(v) {
  return {
    type: "file",
    mimeType: v.mimeType,
    data: v.encoding === "path" ? (0, import_fs.readFileSync)(v.data) : v.data
  };
}
var userMsg = (template, ...values) => (0, import_resolved3.pipe)((0, import_text.t)(template, ...values))._(import_text.tParse)._(
  (parts) => parts.map(
    (v) => (0, import_ts_pattern.match)(v).with(import_ts_pattern.P.string, mapTextPart).with({ type: "image" }, mapImagePart).with({ type: "file" }, mapFilePart).exhaustive()
  )
)._(
  (content) => ({
    role: "user",
    content
  })
).$;
var assistantMsg = (template, ...values) => {
  return (0, import_resolved3.pipe)((0, import_text.t)(template, ...values))._(import_text.tParse)._(
    (parts) => parts.map(
      (v) => (0, import_ts_pattern.match)(v).with(import_ts_pattern.P.string, mapTextPart).with({ type: "tool-call" }, (v2) => v2).exhaustive()
    )
  )._(
    (content) => ({
      role: "assistant",
      content
    })
  ).$;
};
var systemMsg = (template, ...values) => {
  return (0, import_resolved3.pipe)((0, import_text.t)(template, ...values))._(
    (content) => ({ role: "system", content })
  ).$;
};

// src/part.builder.ts
var import_mime_types = __toESM(require("mime-types"), 1);
var filePart = {
  fromPath(path, mimeType) {
    const extractedMimeType = mimeType ?? import_mime_types.default.lookup(path);
    if (!extractedMimeType)
      throw new Error(
        `Mimetype cannot be inferred for file ${path}, provide it as a parameter instead`
      );
    if (extractedMimeType.startsWith("image/"))
      return {
        type: "image",
        encoding: "path",
        mimeType,
        data: path
      };
    else
      return {
        type: "file",
        encoding: "path",
        mimeType,
        data: path
      };
  },
  fromBase64(base64, mimeType) {
    const extractedMimeType = mimeType ?? base64.match(/^data:([^;]+);/)?.[1];
    if (!extractedMimeType)
      throw new Error(
        `Mimetype cannot be inferred for base64 ${base64}, provide it as a parameter instead`
      );
    if (extractedMimeType.startsWith("image/"))
      return {
        type: "image",
        encoding: "base64",
        mimeType,
        data: base64
      };
    else
      return {
        type: "file",
        encoding: "base64",
        mimeType,
        data: base64
      };
  },
  fromUrl(url, mimeType) {
    const extractedMimeType = mimeType ?? import_mime_types.default.lookup(url);
    if (!extractedMimeType)
      throw new Error(
        `Mimetype cannot be inferred for url ${url}, provide it as a parameter instead`
      );
    return {
      type: extractedMimeType.startsWith("image/") ? "image" : "file",
      encoding: "url",
      mimeType,
      url
    };
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  CompletionBuilder,
  assistantMsg,
  completion,
  filePart,
  systemMsg,
  userMsg
});
//# sourceMappingURL=llm.index.cjs.map