// src/model.middleware.ts
import "@synstack/fs-cache";
import { experimental_wrapLanguageModel } from "ai";

// src/message.builder.ts
import { pipe } from "@synstack/resolved";
import { match, P } from "ts-pattern";
import { t, tParse } from "@synstack/text";
import { readFileSync } from "fs";
var messageToText = (message) => {
  if (typeof message.content === "string") return message.content;
  return message.content.reduce((v, c) => {
    if (c.type === "text") return v + c.text;
    return v;
  }, "");
};

// src/model.middleware.ts
var includeAssistantMessage = (model) => experimental_wrapLanguageModel({
  model,
  middleware: {
    wrapGenerate: async ({ doGenerate, params }) => {
      const lastMessage = params.prompt.at(-1);
      const result = await doGenerate();
      if (lastMessage?.role === "assistant") {
        return {
          ...result,
          text: messageToText(lastMessage) + (result.text ?? "")
        };
      }
      return result;
    },
    wrapStream: async ({ doStream, params }) => {
      const lastMessage = params.prompt.at(-1);
      const result = await doStream();
      if (lastMessage?.role !== "assistant") {
        return result;
      }
      const outputStream = result.stream.pipeThrough(
        new TransformStream({
          start(controller) {
            controller.enqueue({
              type: "text-delta",
              textDelta: messageToText(lastMessage)
            });
          }
        })
      );
      return {
        rawCall: result.rawCall,
        rawResponse: result.rawResponse,
        request: result.request,
        warnings: result.warnings,
        stream: outputStream
      };
    }
  }
});
var cacheCalls = (cache) => (model) => {
  return experimental_wrapLanguageModel({
    model,
    middleware: {
      wrapGenerate: async ({ doGenerate, params }) => {
        const [hit, value] = await cache.get([params]);
        if (hit === "hit") {
          return value;
        }
        const res = await doGenerate();
        await cache.set([params], res);
        return res;
      },
      wrapStream: async ({ doStream, params }) => {
        const [hit, value] = await cache.get([params]);
        if (hit === "hit") {
          let index = 0;
          return {
            ...value,
            stream: new ReadableStream({
              pull(controller) {
                if (index < value.stream.length) {
                  controller.enqueue(value.stream[index]);
                  index++;
                } else {
                  controller.close();
                }
              }
            })
          };
        }
        const res = await doStream();
        const collector = [];
        res.stream.pipeThrough(
          new TransformStream({
            transform(chunk, controller) {
              collector.push(chunk);
              controller.enqueue(chunk);
            }
          })
        );
        await cache.set([params], {
          ...res,
          stream: collector
        });
        return res;
      }
    }
  });
};
export {
  cacheCalls,
  includeAssistantMessage
};
//# sourceMappingURL=middleware.index.js.map